
; ---------------------------------------------------------------------------------------------	;
;	    Touch Memory.						;
;	    Dallas-1990A      .		;
;	       EEPROM   00-07.			;
; ---------------------------------------------------------------------------------------------	;

	errorlevel 1

	processor pic16f628

	#include <P16F628.INC>

	__CONFIG H'3F10'	;       "",    !
				;    _WDT_OFF & _PWRTE_ON & _MCLRE_ON  .  ...))


; -------------------------   ---------------------------------------------

RD_COUNT	equ 0x20
LONGCOUNT	equ 0x21
CRC_COUNTER	equ 0x22
CRC_TEMP	equ 0x23
SYMBADR_L	equ 0x24
SYMBADR_H	equ 0x25
BYTE_COUNT	equ 0x26
SYMBOL		equ 0x27

;--------------------------       -----------------------------

KEY		equ 0x70	;      ,   0x70 - 0x77.

FLAGS		equ 0x78
CRC		equ 0x79
WRBUFFER	equ 0x7A
DELCOUNT	equ 0x7B
TEMP		equ 0x7C
TEMP2		equ 0x7D

;-----------------------------------------------------------------------------------------------

#define IN_KEY   PORTA, 2	;   1-Wire.
#define OUT_KEY  TRISA, 2	;   1-Wire.

#define PFLAG    FLAGS, 0	;     .
#define GOODCODE FLAGS, 1	;  ,      (   ).
#define GOODKEY  FLAGS, 2	;  ,       = 0




; --------------------------------------------------------------------------------------------------------
;  
; --------------------------------------------------------------------------------------------------------

RESET	movlw 7			;      /,  .
	movwf CMCON

	movlw 0x93		;   -  .   -
	movwf PORTA		;     LCD  .

	clrf PORTB		;   -  .

	bsf STATUS, RP0		;       1.

	movlw 0x5F		;   ,  TMR0  
	movwf OPTION_REG	;  .      !!!

	movlw 0x2C		;  .
	movwf TRISA
	movlw 6
	movwf TRISB

	bcf STATUS, RP0		;     0.

	clrf FLAGS		;      .



; -------------------------------------------------------------------------------------------------------
;    LCD,       
; -------------------------------------------------------------------------------------------------------

	movlw d'20'		;     50 , 
	call LONGDELAY

	movlw 0x30		;   , 8- .
	call SENDINIT

	movlw 2			;  5 , 
	call LONGDELAY

	movlw 0x30		;    , 8- .
	call SENDINIT

	movlw d'20'		;  100 ,  
	call DELAY

	movlw 0x30		;     , 8- .
	call SENDINIT

	movlw 8			;  40 .
	call DELAY

	movlw 0x20		;    4-  .
	call SENDINIT

	movlw 0xD8		;   TMR0  40      .
	movwf TMR0		;  ,    . .  = 40 .
	bcf INTCON, T0IF

	movlw 0x28		;    4-  , 2 , 0- 
	call SENDCOM		; .

	movlw 0x0C		;  ,   .
	call SENDCOM

	movlw 6			;     .
	call SENDCOM

	movlw 0			;     .
	call MESSAGE_HIGH
	movlw 1
	call MESSAGE_LOW

	movlw 0			;      2/3 .
	call LONGDELAY
	movlw 0
	call LONGDELAY

	movlw 0x90		;      .
	movwf PORTA



; ---------------------------------------------------------------------------------------
;    
; ---------------------------------------------------------------------------------------


RD_CLR	call CLEANPOINT		;   ,  .

RD_1	movlw 2			;   " ".
	call MESSAGE_HIGH

RD_WAIT	call SEND_RESET		;     "RESET".       
	btfss PFLAG		;   PRESENCE,  PFLAG  . 
	goto RD_WAIT		;   .

	call READ_KEY		;   .

	btfss GOODCODE		;    (CRC  =0),    .
	goto RD_ERR

; ------------------------

RD_OKAY	movlw 3			;   " ".
	call MESSAGE_HIGH

	call SENDKEY		;     .

	movlw 0x80		;     .
	movwf TEMP

RD_BEEP	movlw 0x82		;    .   ,  ,
	movwf PORTA		;   .

	movlw d'100'		; .
	call DELAY

	movlw 0x92		;   ,  ,    .
	movwf PORTA

	movlw d'100'		; .
	call DELAY

	decfsz TEMP, f		;  .  ,     .
	goto RD_BEEP

	movlw d'70'		;   ...
	call LONGDELAY

	movlw 0x91		;      .
	movwf PORTA

	movlw 0			;  ,   1,5 .
	call LONGDELAY
	movlw 0x80
	call LONGDELAY

	call WR_EEPROM		;     EEPROM.     - !

	goto RD_1		;   ,    .

; ------------------------

RD_ERR	movlw 4			;   " !".
	call MESSAGE_HIGH

	call CLEANPOINT		;   ,   .

	movlw 0x90		;   -     .
	movwf PORTA

	movlw 0			; ,   1 .
	call LONGDELAY

	goto RD_1



; --------------------------------------------------------------------------------------------------
; .
; --------------------------------------------------------------------------------------------------


;         .
;       2-  SYMBADR_H, SYMBADR_L.   
;    ,       
; .  ,        ,
;        ( 256 ).

GETSYMBOL
	movlw high MESS_TABLE
	addwf SYMBADR_H, w	
	movwf PCLATH

	movlw low MESS_TABLE
	addwf SYMBADR_L, w

	btfsc STATUS, C
	incf PCLATH, f

	movwf PCL


MESS_TABLE
	retlw 0xA8		; 0 = " "
	retlw A'p'
	retlw A'o'
	retlw 0xB4
	retlw A'p'
	retlw A'a'
	retlw 0xBC
	retlw 0xBC
	retlw A'a'
	retlw A' '
	retlw 0xC0
	retlw 0xBF
	retlw A'e'
	retlw 0xBD
	retlw 0xB8
	retlw 0xC7

	retlw A' '
	retlw 0xBA		; 1 = " DS1990A"
	retlw 0xBB
	retlw 0xC6
	retlw 0xC0
	retlw A'e'
	retlw 0xB9
	retlw A' '
	retlw A'D'
	retlw A'S'
	retlw A'1'
	retlw A'9'
	retlw A'9'
	retlw A'0'
	retlw A'A'
	retlw A' '

	retlw A' '		; 2 = " "
	retlw 0xA8
	retlw A'p'
	retlw 0xB8
	retlw 0xBB
	retlw A'o'
	retlw 0xB6
	retlw 0xB8
	retlw 0xBF
	retlw A'e'
	retlw A' '
	retlw 0xBA
	retlw 0xBB
	retlw 0xC6
	retlw 0xC0
	retlw A' '

	retlw A' '		; 3 = " !"
	retlw A'K'
	retlw 0xBB
	retlw 0xC6
	retlw 0xC0
	retlw A' '
	retlw 0xBE
	retlw A'p'
	retlw A'o'
	retlw 0xC0
	retlw 0xB8
	retlw 0xBF
	retlw A'a'
	retlw 0xBD
	retlw A'!'
	retlw A' '

	retlw A' '		; 4 = " !"
	retlw A'O'
	retlw 0xC1
	retlw 0xB8
	retlw 0xB2
	retlw 0xBA
	retlw A'a'
	retlw A' '
	retlw 0xC0
	retlw 0xBF
	retlw A'e'
	retlw 0xBD
	retlw 0xB8
	retlw 0xC7
	retlw A'!'
	retlw A' '

; ------------------------------------------------------------------------------------------



;    .       WREG.
;      ,    (16 ),
;       16.
;   2    : MESSAGE_HIGH -    
;   MESSAGE_LOW -  .


MESSAGE_HIGH
	movwf SYMBADR_L		;   .

	movlw 0x80		;   WREG      
	goto MESSAGE		;  .

MESSAGE_LOW
	movwf SYMBADR_L		;   .

	movlw 0xC0		;         .

MESSAGE	call SENDCOM		;       .

	swapf SYMBADR_L, f	;      .
	clrf SYMBADR_H

	movlw 0x10		;      16.
	movwf BYTE_COUNT

MS_CYC	call GETSYMBOL		;     .

	call SENDSYMBOL		;    .

	incf SYMBADR_L, f	;     .

	decfsz BYTE_COUNT, f	;       .
	goto MS_CYC

	return			;   .

; --------------------------------------------------------------------------------------------------



;       .
;     W.

SENDSYMBOL
	movwf SYMBOL		;   .


SYMB_W	btfss INTCON, T0IF	;     .   ,
	goto SYMB_W		;   40 ,       .

	andlw 0xF0		;    , RS=1, E=1.
	iorlw 9
	movwf PORTB

	andlw 0xFE		;  , E=0.
	movwf PORTB

	swapf SYMBOL, w		;    , RS=1, E=1.
	andlw 0xF0
	iorlw 9
	movwf PORTB

	andlw 0xFE		;  , E=0.
	movwf PORTB

	movlw 0xD8		;      
	movwf TMR0		;   = 40 ,    .
	bcf INTCON, T0IF

	return

; ----------------------------------------------------------------------------------------------



;    .
;       W.

SENDCOM
	movwf SYMBOL		;   .


COM_W	btfss INTCON, T0IF	;     .   ,
	goto COM_W		;   40 ,       .

	andlw 0xF0		;    , RS=0, E=1.
	iorlw 1
	movwf PORTB

	andlw 0xFE		;  , E=0.
	movwf PORTB

	swapf SYMBOL, w		;    , RS=0, E=1.
	andlw 0xF0
	iorlw 1
	movwf PORTB

	andlw 0xFE		;  , E=0.
	movwf PORTB

	movlw 0xD8		;      
	movwf TMR0		;   = 40 ,    .
	bcf INTCON, T0IF


	return

;----------------------------------------------------------------------------------------------



;      8- ,   ,
;      W.    .. = 0000.


SENDINIT

	iorlw 1			;       , RS=0, E=1,
	movwf PORTB		;   SYMBOL.

	andlw 0xFE		;  , E=0.
	movwf PORTB

	return

;---------------------------------------------------------------------------------------------



;       KEY  .
;     .

SENDKEY	movlw 0xC0		;     -  
	call SENDCOM		;   2- .

	movlw KEY+7		;  FSR    , 
	movwf FSR		;   .

	movlw d'16'		;     -  
	movwf BYTE_COUNT	; .

SENDK_1	swapf INDF, f		;     ,  
	movlw 0x0F		;  FSR,     
	andwf INDF, w		;       .
	sublw 9
	btfss STATUS, C
	addlw 0xF9
	sublw 0x39
	call SENDSYMBOL

	btfsc BYTE_COUNT, 0	;   2-    FSR  1
	decf FSR, f

	decfsz BYTE_COUNT, f	;  .
	goto SENDK_1

	return

; ---------------------------------------------------------------------------------------------------



;      16  "." .

CLEANPOINT

	movlw 0xC0		;       .
	call SENDCOM

	movlw 0x10		;     16.
	movwf TEMP

CL_LOOP	movlw 0xDF		;       "."
	call SENDSYMBOL
	decfsz TEMP, f
	goto CL_LOOP

	return			; ,   .

; ------------------------------------------------------------------------------------------------



 ;    "Reset"    "Presence".
 ;      "Presence",   PFLAG=1,    = 0.


SEND_RESET

	bcf PFLAG		;   

	btfss IN_KEY		;     =0 (, PRESENCE  ),
	return			;       .

	bsf STATUS, RP0		;    .  = 0.
        bcf OUT_KEY

        movlw d'100'		;   500 
	call DELAY

        bsf OUT_KEY		;  .

	bcf STATUS, RP0		;    0- .

	movlw d'13'		;   70 
	call DELAY

	btfss IN_KEY		;     0,  ,  . 
	bsf PFLAG		;  .   

	movlw d'100'		;     500 . 
        call DELAY		;        . 

	btfss IN_KEY		;     0,    ,    - ,
	bcf PFLAG		;    .      .

	return			; .

 ; -----------------------------------------------------------------------



 ;   .      KEY.
 ;  GOODCODE     .


READ_KEY
	movlw 0x33		;       .
	call WRITE_BYTE

	movlw KEY		;    .
	movwf FSR

	bsf GOODCODE	  	;     .
	bcf GOODKEY		;     GOODKEY .
	clrf CRC		;      


RD_BTN	call READ_BYTE		;  .     . 

	movwf INDF		;     .

	movf INDF, f		;   .   
	btfss STATUS, Z		;   0,    GOODKEY.
	bsf GOODKEY

	call DO_CRC		;  CRC.

	incf FSR, f
	btfss FSR, 3
	goto RD_BTN		;   .

	movf CRC, f		;     CRC .. = 0.    
	btfsc STATUS, Z		; ,        GOODKEY
	btfss GOODKEY		; ( ,     = 0),  
	bcf GOODCODE		;   .

	return			;   .

; ------------------------------------------------------------------------------------------------



 ;  /    .    
 ;    READ_BYTE,    -  WRITE_BYTE.
 ;     W,     W  WRBUFFER.


READ_BYTE			;     .   "1" 
	movlw 0xFF		;    /

WRITE_BYTE
	movwf WRBUFFER		;     .  
	movlw 8			;   WRBUFFER,   
	movwf RD_COUNT		;   8.

RBLoop	bsf STATUS, RP0		;    1.
	bcf OUT_KEY		; =0.   .
        nop			; 1 
	nop			; 2 
	nop			; 3 
        btfsc WRBUFFER, 0	; 4 
        bsf OUT_KEY		; 5 ,   0 WRBUFFER = 1,   .
        rrf WRBUFFER, f		; 6 ,   .
	bcf STATUS, RP0		; 7 ,     0.
        nop			; 8 
        nop			; 9 
        nop			; 10 
        bcf WRBUFFER, 7		; 12 
        btfsc IN_KEY		; 13  -   .
        bsf WRBUFFER, 7		;       .

        movlw 8			;       60 .
	call DELAY

	bsf STATUS, RP0		;  .  = 1.
        bsf OUT_KEY
	bcf STATUS, RP0

        movlw 4			;    30     .
        call DELAY

        decfsz RD_COUNT, f	;   / 
        goto RBLoop

        movf WRBUFFER, w	;     W.

        return			;   .
 
; -----------------------------------------------------------------------------------------------



 ;     CRC.    
 ;   CRC.      WRBUFFER, 
 ;    ,     (  ).


DO_CRC	movlw 8			;  8   .
	movwf CRC_COUNTER

CRC_1	movf WRBUFFER, w	;   CRC.   ,  
	xorwf CRC, w		;     .     ,
	movwf CRC_TEMP		;   .
	rrf CRC_TEMP, w

	movlw 0x18		;   (   C) =1,   
	btfsc STATUS, C		;  CRC   18h.
	xorwf CRC, f

	rrf CRC, f		;    ,    CRC 
	rrf WRBUFFER, f		;     .  WRBUFFER    .

	decfsz CRC_COUNTER, f	;    .
	goto CRC_1

	return			; ,   .

 ; -----------------------------------------------------------------------------------------



;     KEY,    ,  EEPROM.
;    0- ,    family code, 
;   CRC.


WR_EEPROM
	movlw KEY		;  FSR      .
	movwf FSR

	bsf STATUS, RP0		;   ,     EEPROM.

	clrf EEADR		;    EEPROM (=0).

	bsf EECON1, WREN	;    EEPROM.

WREPR_1	movf INDF, w		;  .        
	movwf EEDATA		;   EEPROM.

	movlw 0x55		;  . .
	movwf EECON2
	movlw 0xAA
	movwf EECON2

	bsf EECON1, WR		;  .

WREPR_2	btfsc EECON1, WR	;    .
	goto WREPR_2

	incf FSR, f		;   FSR    .

	incf EEADR, f		;    EEPROM    1.

	btfss FSR, 3		; ,     .  , 
	goto WREPR_1		;  .

	bcf EECON1, WREN	;  -     EEPROM.

	bcf STATUS, RP0		;   0-  .

	return			;  .

 ; --------------------------------------------------------------------------------------------



;   .      
;    (-4)/5 = .  ..   W.

DELAY	movwf DELCOUNT		;      W.

D_LOOP	nop			;    5-    .
	nop
	decfsz DELCOUNT, f
	goto D_LOOP

	return			; ,   .

; -------------------------------------------------------------------------------------------------



;    .
;    W,    = 2560*W 


LONGDELAY
	movwf LONGCOUNT		;   .      W.

L_LOOP	movlw 0			;    2-    
	call DELAY		;     .
	movlw 0
	call DELAY
	decfsz LONGCOUNT, f
	goto L_LOOP

	return			; ,   .

; --------------------------------------------------------------------------------------------------



	end			;  .




